require 'cabal/util'
require 'cabal/client'
require 'cabal/identity_manager'
require 'tempfile'
require 'securerandom'
require 'uri'
require 'fileutils'

module Cabal
  # Generates SSH sessions for remote server interaction
  class SSH
    # Create a new instance and use it to generate an SSH session
    # @param options [Hash] the options to pass along to the new instance
    # @option options [String] :cluster the name of the cluster key to
    #   use (required)
    # @option options [String] :ssh_signature the user@hostname
    #   that would be passed if the normal ssh CLI utility were used (required)
    # @option options [Cabal::Client] :client the instance of
    #   Cabal::Client to use for key retrieval (required)
    # @option options [Integer] :identity_timeout the number of seconds
    #   that keys should be forwarded in the resulting SSH session (optional)
    # @option options [Kernel] :kernel the Kernel-compliant object used for
    #   issuing calls to #system for the SSH session (optional)
    # @return [nil]
    def self.connect(options = {})
      new(
        options[:cluster],
        options[:ssh_signature],
        options[:client],
        options[:identity_timeout] || 1800,
        options[:kernel] || Kernel
      ).connect
    end

    def initialize(cluster, ssh_signature, client, identity_timeout, kernel)
      @cluster = cluster
      @ssh_signature = ssh_signature
      @client = client
      @identity_timeout = identity_timeout
      @kernel = kernel
    end

    # The lifetime option that would be passed to ssh-add for this session
    # @return [String] the empty string for immortal sessions,
    #   "-t identity_timeout" for sessions with a defined lifetime
    def lifetime
      return '' if identity_timeout == 0
      "-t #{identity_timeout}"
    end

    # Initiate an SSH connection in the current terminal
    # @return [nil]
    def connect
      begin
        write_private_key
        initiate_connection
      ensure
        terminate_connection
      end
    end

    private
    def write_private_key
      private_key = client.private_key(cluster)
      @key_file = Tempfile.new(SecureRandom.hex(8))
      key_file.write(private_key)
      key_file.close
      FileUtils.chmod(0700, key_file.path)
    end

    def initiate_connection
      agent.start
      kernel.system("#{agent.env} ssh-add #{lifetime} #{key_file.path} > /dev/null 2>&1")
      kernel.system("#{agent.env} ssh -A #{ssh_signature}")
    end

    def terminate_connection
      kernel.system("#{agent.env} ssh-add -D > /dev/null 2>&1")
      agent.stop
      key_file.unlink unless key_file.nil?
    end

    def key_file
      @key_file
    end

    def ssh_signature
      @ssh_signature
    end
    
    def client
      @client
    end

    def cluster
      @cluster
    end

    def agent
      @agent ||= Cabal::IdentityManager.new
    end

    def identity_timeout
      @identity_timeout
    end

    def kernel
      @kernel
    end
  end
end
